-- Samnites Decisions
--=======================================================================================================================
include("PlotIterators.lua")

-- IsUsingCBP
function IsUsingCBP()
	local CBP = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if (mod.ID == CBP) then
			return true
		end
	end
	return false
end
local isUsingCBP = IsUsingCBP()
--=======================================================================================================================
-- Civ Specific Decisions
--=======================================================================================================================
-- Samnites: Regulate the use of the tratturi
-------------------------------------------------------------------------------------------------------------------------
local pasture = GameInfo.Improvements.IMPROVEMENT_PASTURE.ID
local camp = GameInfo.Improvements.IMPROVEMENT_CAMP.ID
local road = GameInfo.Routes.ROUTE_ROAD.ID
local iCulture = GameInfo.Yields.YIELD_CULTURE.ID


local Decisions_SamnitesTratturi = {}
    Decisions_SamnitesTratturi.Name = "TXT_KEY_DECISIONS_TRATTURI"
	Decisions_SamnitesTratturi.Desc = "TXT_KEY_DECISIONS_TRATTURI_DESC"
	HookDecisionCivilizationIcon(Decisions_SamnitesTratturi, "CIVILIZATION_SAMNITES")
	Decisions_SamnitesTratturi.CanFunc = (
	function(pPlayer)
		if pPlayer:GetCivilizationType() ~= GameInfoTypes.CIVILIZATION_SAMNITES then return false, false end
		if load(pPlayer, "Decisions_SamnitesTratturi") == true then
			Decisions_SamnitesTratturi.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_TRATTURI_ENACTED_DESC")
			return false, false, true
		end
		local iCost = math.ceil(250*iMod)
		Decisions_SamnitesTratturi.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_TRATTURI_DESC", iCost)
		if (Teams[pPlayer:GetTeam()]:IsHasTech(GameInfoTypes.TECH_ENGINEERING)) and (pPlayer:GetJONSCulture() >= iCost) then
			return true, true
		else
			return true, false
		end
		if (pPlayer:GetNumResourceAvailable(iMagistrate, false) < 1) then return true, false end
		
		
		return true, true
	end
	)
	
	Decisions_SamnitesTratturi.DoFunc = (
	function(pPlayer)
		local iCost = math.ceil(250*iMod)
		pPlayer:ChangeNumResourceTotal(iMagistrate, -1)
		pPlayer:ChangeJONSCulture(-iCost)
		pPlayer:SetNumFreePolicies(1)
		pPlayer:SetNumFreePolicies(0)
		pPlayer:SetHasPolicy(GameInfoTypes.POLICY_DECISIONS_TRATTURI, true)
		
		-- Spawn a road over every pasture
		for pCity in pPlayer:Cities() do
			for adjacentPlot in PlotAreaSweepIterator(pCity:Plot(), 3, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
				x = adjacentPlot:GetX()
				y = adjacentPlot:GetY()
				if adjacentPlot:GetImprovementType() == pasture and adjacentPlot:GetWorkingCity() == pCity then
					if adjacentPlot:GetRouteType() == -1 then
						adjacentPlot:SetRouteType(road)
					end
				end
			end
		end
		save(pPlayer, "Decisions_SamnitesTratturi", true)
	end
	)
	
	Decisions_SamnitesTratturi.Monitors = {}
	Decisions_SamnitesTratturi.Monitors[GameEvents.BuildFinished] = (	
	function(playerID, plotX, plotY, improvementID)
		local pPlot = Map.GetPlot(plotX, plotY)
		local pPlayer = Players[playerID]
		if load(pPlayer, "Decisions_SamnitesTratturi") == true then
			if improvementID == pasture then
				if pPlot:GetRouteType() == -1 then
					pPlot:SetRouteType(road)
					print("EnD addons Tratturi improvement built")
				end
			else
			end
		end
	end
	)
--[[
	Decisions_SamnitesTratturi.Monitors[Events.SerialEventRoadCreated] = (	
	function(playerID, plotX, plotY, improvementID)
		local pPlot = Map.GetPlot(plotX, plotY)
		local pPlayer = Players[playerID]
		if load(pPlayer, "Decisions_SamnitesTratturi") == true then
			if improvementID == pasture or improvementID == camp then
				if pPlot:GetRouteType() == road then
					if pPlayer:IsHuman() then
						Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, 2)
					end
					print("EnD addons TratturiCulture improvement built")
				end
			else
			end
		end
	end
	)
--]]	
Decisions_AddCivilisationSpecific(GameInfoTypes.CIVILIZATION_SAMNITES, "Decisions_SamnitesTratturi", Decisions_SamnitesTratturi)

-------------------------------------------------------------------------------------------------------------------------
-- Samnites: Institute the Verehia in every city
-------------------------------------------------------------------------------------------------------------------------

local Decisions_SamniteVerehia = {}
    Decisions_SamniteVerehia.Name = "TXT_KEY_DECISIONS_VEREHIA"
	Decisions_SamniteVerehia.Desc = "TXT_KEY_DECISIONS_VEREHIA_DESC"
	HookDecisionCivilizationIcon(Decisions_SamniteVerehia, "CIVILIZATION_SAMNITES")
	Decisions_SamniteVerehia.CanFunc = (
	function(pPlayer)
		if pPlayer:GetCivilizationType() ~= GameInfoTypes.CIVILIZATION_SAMNITES then return false, false end
		if load(pPlayer, "Decisions_SamniteVerehia") == true then
			Decisions_SamniteVerehia.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_VEREHIA_ENACTED_DESC")
			return false, false, true
		end
		
		local iCost = math.ceil(300 * iMod)
		Decisions_SamniteVerehia.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_VEREHIA_DESC", iCost)
		
		if (pPlayer:GetNumResourceAvailable(iMagistrate, false) < 1) then return true, false end

		if pPlayer:GetGold() >= iCost then
			return true, true
		else 
			return true, false
		end
			
		return true, true
	end
	)
	
	Decisions_SamniteVerehia.DoFunc = (
	function(pPlayer)
		local iCost = math.ceil(300 * iMod)
		pPlayer:ChangeNumResourceTotal(iMagistrate, -1)
		pPlayer:ChangeGold(-iCost)
		pPlayer:SetNumFreePolicies(1)
		pPlayer:SetNumFreePolicies(0)
		pPlayer:SetHasPolicy(GameInfoTypes.POLICY_DECISIONS_VEREHIA, true)
		save(pPlayer, "Decisions_SamniteVerehia", true)
	end
	)
	
	Decisions_SamniteVerehia.Monitors = {}
	Decisions_SamniteVerehia.Monitors[Events.SerialEventCityPopulationChanged] = (	
	function(x, y, newPop)
		local pPlot = Map.GetPlot(ToGridFromHex(x, y))
		local pCity = pPlot:GetPlotCity()
		local pPlayer = Players[pCity:GetOwner()]
		local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
		local bonus = math.ceil(20 * iMod)
		if load(pPlayer, "Decisions_SamniteVerehia") == true then
			if (not isUsingCBP) and pPlayer:IsHuman() then
				pCity:ChangeProduction(bonus)
				pCity:ChangeJONSCultureLevel(bonus)
				-- or pCity:ChangeJONSCultureStored(bonus)
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), "[ICON_CITIZEN]! [COLOR_CULTURE_STORED]+" .. bonus .. "[ICON_CULTURE][ENDCOLOR]", 2)
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), "[ICON_CITIZEN]! [COLOR_YIELD_FOOD]+" .. bonus .. "[ICON_PRODUCTION][ENDCOLOR]", 3)
			else
			end
		end
	end
	)
	
	Decisions_SamniteVerehia.Monitors[GameEvents.UnitSetXY] = (	
	function(iPlayer, iUnitID, iX, iY)
		local pPlayer = Players[iPlayer]
		local plot = Map.GetPlot(iX, iY);
		local pUnit = pPlayer:GetUnitByID(iUnitID);
		local iStrength = GameInfo.Units[pUnit:GetUnitType()].Combat
		if load(pPlayer, "Decisions_SamniteVerehia") == true then
			if pUnit:IsGarrisoned() then
				pUnit:SetHasPromotion(GameInfoTypes["PROMOTION_DECISIONS_VEREHIA"], true);
			else
				pUnit:SetHasPromotion(GameInfoTypes["PROMOTION_DECISIONS_VEREHIA"], false);
			end
		end
	end
	)
	
Decisions_AddCivilisationSpecific(GameInfoTypes.CIVILIZATION_SAMNITES, "Decisions_SamniteVerehia", Decisions_SamniteVerehia)


print("Samnites Decisions: loaded")