-- Jarcast Samnites Functions
-- Author: Jarcast
--=======================================================================================================================
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
WARN_NOT_SHARED = false; include( "SaveUtils" ); MY_MOD_NAME = "Jarcast_Samnites";

include("FLuaVector.lua")
include("IconSupport.lua")
include("InstanceManager.lua")
include("PlotIterators.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local Teams 					= Teams
local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function Neirai_GetNearestCity(pPlayer, pPlot)
    local distance = 9999
    local cNearestCity = nil
    for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pPlot:GetX(), pPlot:GetY())
        if between < distance then
            distance = between
            cNearestCity = cCity
        end
    end
    return cNearestCity
end
-------------------------------------------------------------------------------------------------------------------------
function GetBuildIDfromImprovementID(improvID)
	for row in DB.Query("Select a.ID ImprovID, a.Type ImprovTipe, b.ID BuildID, b.Type from Improvements a, Builds b WHERE a.Type = b.ImprovementType and a.ID = '" .. improvID .. "'") do
		return row.BuildID
	end
end
--=======================================================================================================================
-- Globals
--=======================================================================================================================
local iCiv = GameInfoTypes.CIVILIZATION_SAMNITES
local traitSamnitesID = GameInfoTypes["TRAIT_SAMNITES"]
local g_IsTraitActive = IsTraitActive(traitSamnitesID)
local iFaith = GameInfo.Yields.YIELD_FAITH.ID
local iFood = GameInfo.Yields.YIELD_FOOD.ID
local iProduction = GameInfo.Yields.YIELD_PRODUCTION.ID
local iGold = GameInfo.Yields.YIELD_GOLD.ID
local iCulture = GameInfo.Yields.YIELD_CULTURE.ID
if isUsingCBP then
local iTourism = GameInfo.Yields.YIELD_TOURISM.ID -- CBP only
end
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)

local HorseID 	= GameInfoTypes.RESOURCE_HORSE
local CattleID 	= GameInfoTypes.RESOURCE_COW
local BisonID 	= GameInfoTypes.RESOURCE_BISON
local SheepID 	= GameInfoTypes.RESOURCE_SHEEP
local DeerID	= GameInfoTypes.RESOURCE_DEER
local IvoryID	= GameInfoTypes.RESOURCE_IVORY
local FurID		= GameInfoTypes.RESOURCE_FUR
local TrufflesID= GameInfoTypes.RESOURCE_TRUFFLES

local iTotemHorse	= GameInfoTypes["BUILDING_TOTEM_HORSE"]
local iTotemCow		= GameInfoTypes["BUILDING_TOTEM_COW"]
local iTotemBison	= GameInfoTypes["BUILDING_TOTEM_BISON"]
local iTotemSheep	= GameInfoTypes["BUILDING_TOTEM_SHEEP"]
local iTotemDeer	= GameInfoTypes["BUILDING_TOTEM_DEER"]
local iTotemIvory	= GameInfoTypes["BUILDING_TOTEM_IVORY"]
local iTotemFur		= GameInfoTypes["BUILDING_TOTEM_FUR"]
local iTotemTruffles= GameInfoTypes["BUILDING_TOTEM_TRUFFLE"]
local iTotemBird	= GameInfoTypes["BUILDING_TOTEM_BIRD"]

local perset = 0
--=======================================================================================================================
-- Samnite UA: Ver Sacrum (Sacred Spring)
--=======================================================================================================================
function SamnitesUA(iPlayer, iCityX, iCityY)
	local pPlayer = Players[iPlayer]
	if HasTrait(pPlayer, traitSamnitesID) and pPlayer:IsAlive() then
		local iTeam = pPlayer:GetTeam()
		local pPlot = Map.GetPlot(iCityX, iCityY)
		local pCity = pPlot:GetPlotCity()
		local iCity = pCity:GetID()
		
		local nhorse = 0
		local ncow = 0
		local nsheep = 0
		local nbison = 0
		local ndeer = 0
		local nfur = 0
		local nivory = 0
		local ntruffles = 0
	
		-- Part 1: Count how many animal resources are around the city and claim the tiles
		for adjacentPlot in PlotAreaSweepIterator(pPlot, 3, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
			x = adjacentPlot:GetX()
			y = adjacentPlot:GetY()
			
			if adjacentPlot:GetOwner() == -1 or adjacentPlot:GetOwner() == iPlayer then
				if (adjacentPlot:GetResourceType() == HorseID) then
				nhorse = nhorse + 1 --adjacentPlot:GetNumResource()
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				elseif (adjacentPlot:GetResourceType() == CattleID) then
				ncow = ncow +1
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				elseif (adjacentPlot:GetResourceType() == BisonID) then
				nbison = nbison +1
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				elseif (adjacentPlot:GetResourceType() == SheepID) then
				nsheep = nsheep +1
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				elseif (adjacentPlot:GetResourceType() == DeerID) then
				ndeer = ndeer +1
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				elseif (adjacentPlot:GetResourceType() == IvoryID) then
				nivory = nivory +1
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				elseif (adjacentPlot:GetResourceType() == FurID) then
				nfur = nfur +1
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				elseif (adjacentPlot:GetResourceType() == TrufflesID) then
				ntruffles = ntruffles +1
					if adjacentPlot:GetOwner() == -1 then
						adjacentPlot:SetOwner(iPlayer, iCity, true, true);
						adjacentPlot:SetRevealed(iTeam, true)
					end
				else
				end
			end
		end
		-- Part 2: Depending on the quantity and quality of the animal resources, choses a sacred animal
		-- and gives to the city a dummy building with the iTotem benefits
		if pPlayer:IsHuman() then
			local bcheck = nhorse>0 or ncow>0 or nbison>0 or nsheep>0 or ndeer>0 or nivory>0 or nfur>0 or ntruffles>0
			if bcheck then
				Controls.ButtonHorse:SetHide(false)
				Controls.ButtonCow:SetHide(false)
				Controls.ButtonBison:SetHide(false)
				Controls.ButtonSheep:SetHide(false)
				Controls.ButtonDeer:SetHide(false)
				Controls.ButtonIvory:SetHide(false)
				Controls.ButtonFur:SetHide(false)
				Controls.ButtonTruffles:SetHide(false)
			
				if nhorse < 1 then Controls.ButtonHorse:SetHide(true) end
				if ncow < 1 then Controls.ButtonCow:SetHide(true) end
				if nbison < 1 then Controls.ButtonBison:SetHide(true) end
				if nsheep < 1 then Controls.ButtonSheep:SetHide(true) end
				if ndeer < 1 then Controls.ButtonDeer:SetHide(true) end
				if nivory < 1 then Controls.ButtonIvory:SetHide(true) end
				if nfur < 1 then Controls.ButtonFur:SetHide(true) end
				if ntruffles < 1 then Controls.ButtonTruffles:SetHide(true) end
				Controls.ButtonStack:CalculateSize();
				Controls.TotemStackFrame:DoAutoSize();
				
				local tNearbyanimals = {}
				table.insert(tNearbyanimals, nhorse)
				table.insert(tNearbyanimals, ncow)
				table.insert(tNearbyanimals, nbison)	
				table.insert(tNearbyanimals, nsheep)	
				table.insert(tNearbyanimals, ndeer)	
				table.insert(tNearbyanimals, nivory)	
				table.insert(tNearbyanimals, nfur)	
				table.insert(tNearbyanimals, ntruffles) 	
				save( pPlot, "NearbyAnimals", tNearbyanimals );				
				save(pPlayer,"tSamniteCity", pCity)
				
				if (ContextPtr:IsHidden()) then
					ContextPtr:SetHide(false)
				end
			else
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, 2 )
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
				pCity:SetNumRealBuilding(iTotemBird, 1)			
			end		
		else
			local probhorse = nhorse * GetRandom(1, 1000)
			local probcow = ncow * GetRandom(1, 1000)
			local probsheep = nsheep * GetRandom(1, 1000)
			local probbison = nbison * GetRandom(1, 1000)
			local probdeer = ndeer * GetRandom(1, 1000)
			local probivory = nivory * GetRandom(1, 1000) *2
			local probfur = nfur * GetRandom(1, 1000) *2
			local probtruffles = ntruffles * GetRandom(1, 1000) *2
			local probmax = math.max(probhorse,probcow,probsheep,probbison,probdeer,probivory,probfur,probtruffles)
			if  probmax > 0 then
				if  probmax == probhorse and not pCity:IsHasBuilding(iTotemHorse)then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, nhorse )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 1)
					pCity:SetNumRealBuilding(iTotemCow, 0)
					pCity:SetNumRealBuilding(iTotemBison, 0)
					pCity:SetNumRealBuilding(iTotemSheep, 0)
					pCity:SetNumRealBuilding(iTotemDeer, 0)
					pCity:SetNumRealBuilding(iTotemIvory, 0)
					pCity:SetNumRealBuilding(iTotemFur, 0)
					pCity:SetNumRealBuilding(iTotemTruffles, 0)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				elseif  probmax == probcow and not pCity:IsHasBuilding(iTotemCow)then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, ncow )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 0)
					pCity:SetNumRealBuilding(iTotemCow, 1)
					pCity:SetNumRealBuilding(iTotemBison, 0)
					pCity:SetNumRealBuilding(iTotemSheep, 0)
					pCity:SetNumRealBuilding(iTotemDeer, 0)
					pCity:SetNumRealBuilding(iTotemIvory, 0)
					pCity:SetNumRealBuilding(iTotemFur, 0)
					pCity:SetNumRealBuilding(iTotemTruffles, 0)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				elseif  probmax == probsheep and not pCity:IsHasBuilding(iTotemSheep)then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, nsheep )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 0)
					pCity:SetNumRealBuilding(iTotemCow, 0)
					pCity:SetNumRealBuilding(iTotemBison, 0)
					pCity:SetNumRealBuilding(iTotemSheep, 1)
					pCity:SetNumRealBuilding(iTotemDeer, 0)
					pCity:SetNumRealBuilding(iTotemIvory, 0)
					pCity:SetNumRealBuilding(iTotemFur, 0)
					pCity:SetNumRealBuilding(iTotemTruffles, 0)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				elseif  probmax == probbison and not pCity:IsHasBuilding(iTotemBison) then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, nbison )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 0)
					pCity:SetNumRealBuilding(iTotemCow, 0)
					pCity:SetNumRealBuilding(iTotemBison, 1)
					pCity:SetNumRealBuilding(iTotemSheep, 0)
					pCity:SetNumRealBuilding(iTotemDeer, 0)
					pCity:SetNumRealBuilding(iTotemIvory, 0)
					pCity:SetNumRealBuilding(iTotemFur, 0)
					pCity:SetNumRealBuilding(iTotemTruffles, 0)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				elseif  probmax == probdeer and not pCity:IsHasBuilding(iTotemDeer)then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, ndeer )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 0)
					pCity:SetNumRealBuilding(iTotemCow, 0)
					pCity:SetNumRealBuilding(iTotemBison, 0)
					pCity:SetNumRealBuilding(iTotemSheep, 0)
					pCity:SetNumRealBuilding(iTotemDeer, 1)
					pCity:SetNumRealBuilding(iTotemIvory, 0)
					pCity:SetNumRealBuilding(iTotemFur, 0)
					pCity:SetNumRealBuilding(iTotemTruffles, 0)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				elseif  probmax == probivory and not pCity:IsHasBuilding(iTotemIvory)then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, nivory )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 0)
					pCity:SetNumRealBuilding(iTotemCow, 0)
					pCity:SetNumRealBuilding(iTotemBison, 0)
					pCity:SetNumRealBuilding(iTotemSheep, 0)
					pCity:SetNumRealBuilding(iTotemDeer, 0)
					pCity:SetNumRealBuilding(iTotemIvory, 1)
					pCity:SetNumRealBuilding(iTotemFur, 0)
					pCity:SetNumRealBuilding(iTotemTruffles, 0)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				elseif  probmax == probfur and not pCity:IsHasBuilding(iTotemFur)then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, nfur )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 0)
					pCity:SetNumRealBuilding(iTotemCow, 0)
					pCity:SetNumRealBuilding(iTotemBison, 0)
					pCity:SetNumRealBuilding(iTotemSheep, 0)
					pCity:SetNumRealBuilding(iTotemDeer, 0)
					pCity:SetNumRealBuilding(iTotemIvory, 0)
					pCity:SetNumRealBuilding(iTotemFur, 1)
					pCity:SetNumRealBuilding(iTotemTruffles, 0)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				elseif  probmax == probtruffles and not pCity:IsHasBuilding(iTotemTruffles)then
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, ntruffles )
					Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
					pCity:SetNumRealBuilding(iTotemHorse, 0)
					pCity:SetNumRealBuilding(iTotemCow, 0)
					pCity:SetNumRealBuilding(iTotemBison, 0)
					pCity:SetNumRealBuilding(iTotemSheep, 0)
					pCity:SetNumRealBuilding(iTotemDeer, 0)
					pCity:SetNumRealBuilding(iTotemIvory, 0)
					pCity:SetNumRealBuilding(iTotemFur, 0)
					pCity:SetNumRealBuilding(iTotemTruffles, 1)
					pCity:SetNumRealBuilding(iTotemBird, 0)
				end
			else
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFaith, 2 )
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, perset )
				pCity:SetNumRealBuilding(iTotemHorse, 0)
				pCity:SetNumRealBuilding(iTotemCow, 0)
				pCity:SetNumRealBuilding(iTotemBison, 0)
				pCity:SetNumRealBuilding(iTotemSheep, 0)
				pCity:SetNumRealBuilding(iTotemDeer, 0)
				pCity:SetNumRealBuilding(iTotemIvory, 0)
				pCity:SetNumRealBuilding(iTotemFur, 0)
				pCity:SetNumRealBuilding(iTotemTruffles, 0)
				pCity:SetNumRealBuilding(iTotemBird, 1)
			end
		end
	end
end

if g_IsTraitActive then GameEvents.PlayerCityFounded.Add(SamnitesUA) end

-- No free warrior at beginning to avoid it gets iTotem promotions from cities other than the capital
-- For this reason the first settler only can withdraw melee attacks with 100% probability for the first 
function SquishyFirstSettler(iPlayer, iUnitID)
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnitID)
	if HasTrait(pPlayer, traitSamnitesID) and pPlayer:IsAlive() and Game.GetElapsedGameTurns() < 1 then
		if pUnit:IsHasPromotion(GameInfoTypes["PROMOTION_SACRATUS"]) then		
			pUnit:SetHasPromotion(GameInfoTypes["PROMOTION_FIRSTSACRATUS"], true);
		end
	end
end

if g_IsTraitActive then GameEvents.UnitSetXY.Add(SquishyFirstSettler) end

function SanctuaryAdjustments(iPlayer)
	local pPlayer = Players[iPlayer]
	local currturn = Game.GetElapsedGameTurns()
	local iTurns = math.ceil(8*iMod)
	if HasTrait(pPlayer, traitSamnitesID) and pPlayer:IsAlive() then
		for city in pPlayer:Cities() do
			local foundturn = city:GetGameTurnFounded()
			if currturn == (foundturn + 1) then
				if isUsingCBO and city:IsCapital() then
					city:SetNumRealBuilding(GameInfoTypes["BUILDING_UNITADJ"], 1)
					city:SetNumRealBuilding(GameInfoTypes["BUILDING_UNITADJ"], 0)
				else
					city:SetNumRealBuilding(GameInfoTypes["BUILDING_UNITADJ"], 0)
				end
			
				if not isUsingCBP then
					if city:GetWeLoveTheKingDayCounter() == 0 then
						city:SetResourceDemanded(GameInfoTypes.RESOURCE_MARBLE);
						city:SetWeLoveTheKingDayCounter(iTurns);
					else
						city:ChangeWeLoveTheKingDayCounter(iTurns);
					end
				else
				end
					
				if city:IsHasBuilding(iTotemTruffles) and not pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE"].ID) then
					pPlayer:SetNumFreePolicies(1)
					pPlayer:SetNumFreePolicies(0)
					pPlayer:SetHasPolicy(GameInfoTypes["POLICY_TOTEM_TRUFFLE"], true)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "Praise the [ICON_RES_TRUFFLES]!", 2)
				elseif city:IsHasBuilding(iTotemTruffles) and pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE"].ID) and not pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE2"].ID) then
					pPlayer:SetNumFreePolicies(1)
					pPlayer:SetNumFreePolicies(0)
					pPlayer:SetHasPolicy(GameInfoTypes["POLICY_TOTEM_TRUFFLE2"], true)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "Praise the [ICON_RES_TRUFFLES]!", 2)
				elseif city:IsHasBuilding(iTotemTruffles) and pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE2"].ID) and not pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE3"].ID) then
					pPlayer:SetNumFreePolicies(1)
					pPlayer:SetNumFreePolicies(0)
					pPlayer:SetHasPolicy(GameInfoTypes["POLICY_TOTEM_TRUFFLE3"], true)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "Praise the [ICON_RES_TRUFFLES]!", 2)
				elseif city:IsHasBuilding(iTotemTruffles) and pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE3"].ID) and not pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE4"].ID) then
					pPlayer:SetNumFreePolicies(1)
					pPlayer:SetNumFreePolicies(0)
					pPlayer:SetHasPolicy(GameInfoTypes["POLICY_TOTEM_TRUFFLE4"], true)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "Praise the [ICON_RES_TRUFFLES]!", 2)
				elseif city:IsHasBuilding(iTotemTruffles) and pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE4"].ID) and not pPlayer:HasPolicy(GameInfo.Policies["POLICY_TOTEM_TRUFFLE5"].ID) then
					pPlayer:SetNumFreePolicies(1)
					pPlayer:SetNumFreePolicies(0)
					pPlayer:SetHasPolicy(GameInfoTypes["POLICY_TOTEM_TRUFFLE5"], true)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "Praise the [ICON_RES_TRUFFLES]!", 2)
				else
				end

			end
			
			if isUsingCBO then
				local factor = 1 +0.25*(pPlayer:GetCurrentEra())
				if 	  city:IsHasBuilding(iTotemBird) then
					city:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], 100*factor)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+1[ENDCOLOR][ICON_GREAT_MUSICIAN]", 1)
				elseif city:IsHasBuilding(iTotemCow) then
					city:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_ARTIST"], 100*factor)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+1[ENDCOLOR][ICON_GREAT_ARTIST]", 1)
				elseif city:IsHasBuilding(iTotemHorse) then
					city:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_WRITER"], 100*factor)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+1[ENDCOLOR] [ICON_GREAT_WRITER]", 1)
				elseif city:IsHasBuilding(iTotemSheep) then
					city:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MERCHANT"], 100*factor)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+1[ENDCOLOR] [ICON_GREAT_MERCHANT]", 1)
				elseif city:IsHasBuilding(iTotemBison) then
					city:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_ENGINEER"], 100*factor)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+1[ENDCOLOR] [ICON_GREAT_ENGINEER]", 1)
				elseif city:IsHasBuilding(iTotemDeer) then
					city:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_SCIENTIST"], 100*factor)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+1[ENDCOLOR] [ICON_GREAT_SCIENTIST]", 1)
				end		
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(SanctuaryAdjustments) end;


-- Sanctuary effect on Improvements
Jar_SamnitesUAImprovements_BuildFinished_PlayerID = 0
Jar_SamnitesUAImprovements_BuildFinished_X = 0
Jar_SamnitesUAImprovements_BuildFinished_Y = 0
Jar_SamnitesUAImprovements_BuildFinished_Improvement = 0
	
function Jar_SamnitesUAImprovements(playerID, plotX, plotY, improvementID)
	if(Jar_SamnitesUAImprovements_BuildFinished_PlayerID == playerID and Jar_SamnitesUAImprovements_BuildFinished_X == plotX and Jar_SamnitesUAImprovements_BuildFinished_Y == plotY and Jar_SamnitesUAImprovements_BuildFinished_Improvement == improvementID) then
		return;
	end
	-- Store inputs for duplication check
	Jar_SamnitesUAImprovements_BuildFinished_PlayerID = playerID;	
	Jar_SamnitesUAImprovements_BuildFinished_X = plotX;					
	Jar_SamnitesUAImprovements_BuildFinished_Y = plotY;					
	Jar_SamnitesUAImprovements_BuildFinished_Improvement = improvementID;
	
	local pPlot = Map.GetPlot(plotX, plotY)
	local pPlayer = Players[playerID]
	local pCity = pPlot:GetWorkingCity()
	if pCity then
		if pCity:IsHasBuilding(iTotemCow) then
			if improvementID == GameInfoTypes["IMPROVEMENT_FARM"] or improvementID == GameInfoTypes["IMPROVEMENT_PLANTATION"] then
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iProduction, 1 )
			end
		elseif pCity:IsHasBuilding(iTotemDeer) then
			if improvementID == GameInfoTypes["IMPROVEMENT_CAMP"] then
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iCulture, 1 )
			end
		else
		end
	end
end

if g_IsTraitActive and not isUsingCBP then GameEvents.BuildFinished.Add(Jar_SamnitesUAImprovements) end

function Jar_SheepTotemHillFood(playerID, cityID, plotX, plotY, bGold, bFaithOrCulture)
	local pPlayer = Players[playerID]
	local pCity = pPlayer:GetCityByID(cityID)
	if pCity== nil then pCity = pPlot:GetWorkingCity() end
	local bsheep = pCity:GetNumRealBuilding(iTotemSheep) > 0
	if not bsheep then return end
	local pPlot = Map.GetPlot(plotX, plotY)
	if pPlot:IsHills() then
		Game.SetPlotExtraYield(plotX, plotY, iFood, 1 )
	end
end
if g_IsTraitActive and not isUsingCBP then GameEvents.CityBoughtPlot.Add(Jar_SheepTotemHillFood) end
--===========================================================================
--===========================================================================
-- Following code is edited from WHoward's Morindim clan
-- Set up the iTotem icons for the popup
local g_bWasHidden

local poleTotem = GameInfo.Units[GameInfoTypes["UNIT_SETTLER"]]
Controls.IconTotem:SetHide(not IconHookup(poleTotem.PortraitIndex, 80, poleTotem.IconAtlas, Controls.IconTotem))
local horseTotem = GameInfo.Buildings[iTotemHorse]
Controls.IconHorse:SetHide(not IconHookup(horseTotem.PortraitIndex, 45, horseTotem.IconAtlas, Controls.IconHorse))
local cowTotem = GameInfo.Buildings[iTotemCow]
Controls.IconCow:SetHide(not IconHookup(cowTotem.PortraitIndex, 45, cowTotem.IconAtlas, Controls.IconCow))
local bisonTotem = GameInfo.Buildings[iTotemBison]
Controls.IconBison:SetHide(not IconHookup(bisonTotem.PortraitIndex, 45, bisonTotem.IconAtlas, Controls.IconBison))
local sheepTotem = GameInfo.Buildings[iTotemSheep]
Controls.IconSheep:SetHide(not IconHookup(sheepTotem.PortraitIndex, 45, sheepTotem.IconAtlas, Controls.IconSheep))
local deerTotem = GameInfo.Buildings[iTotemDeer]
Controls.IconDeer:SetHide(not IconHookup(deerTotem.PortraitIndex, 45, deerTotem.IconAtlas, Controls.IconDeer))
local ivoryTotem = GameInfo.Buildings[iTotemIvory]
Controls.IconIvory:SetHide(not IconHookup(ivoryTotem.PortraitIndex, 45, ivoryTotem.IconAtlas, Controls.IconIvory))
local furTotem = GameInfo.Buildings[iTotemFur]
Controls.IconFur:SetHide(not IconHookup(furTotem.PortraitIndex, 45, furTotem.IconAtlas, Controls.IconFur))
local trufflesTotem = GameInfo.Buildings[iTotemTruffles]
Controls.IconTruffles:SetHide(not IconHookup(trufflesTotem.PortraitIndex, 45, trufflesTotem.IconAtlas, Controls.IconTruffles))
local birdTotem = GameInfo.Buildings[iTotemBird]
Controls.IconBird:SetHide(not IconHookup(birdTotem.PortraitIndex, 45, birdTotem.IconAtlas, Controls.IconBird))

-- Resize the button stack
Controls.ButtonStack:CalculateSize();
Controls.TotemStackFrame:DoAutoSize();

-- Hook up the buttons
function OnButtonClicked(iTotem)
	pCity = load(activePlayer,"tSamniteCity")
	pCity:SetNumRealBuilding(iTotem, 1)
	local zPlot = pCity:Plot()
	ttNearbyanimals = load( zPlot, "NearbyAnimals");
	Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iCulture, perset )
	
	if iTotem == iTotemHorse then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[1] )
	elseif iTotem == iTotemCow then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[2] ) 
	elseif iTotem == iTotemBison then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[3] ) 
	elseif iTotem == iTotemSheep then
		Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[4] )
		if not isUsingCBP then
			for i = 0, pCity:GetNumCityPlots()-1, 1 do
				local loopPlot = pCity:GetCityIndexPlot(i)
				if loopPlot and loopPlot:GetOwner() == activePlayerID and loopPlot:IsHills() then
					Game.SetPlotExtraYield(loopPlot:GetX(), loopPlot:GetY(), iFood, 1 )
				end
			end
		end
	elseif iTotem == iTotemDeer then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[5] ) 
	elseif iTotem == iTotemIvory then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[6] ) 
	elseif iTotem == iTotemFur then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[7] ) 
	elseif iTotem == iTotemTruffles then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, ttNearbyanimals[8] )
	elseif iTotem == iTotemBird then Game.SetPlotExtraYield(zPlot:GetX(), zPlot:GetY(), iFaith, 2 )
	end
	save( zPlot, "NearbyAnimals", nil );		
	ContextPtr:SetHide(true)
end
Controls.ButtonHorse:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemHorse) end)
Controls.ButtonCow:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemCow) end)
Controls.ButtonBison:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemBison) end)
Controls.ButtonSheep:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemSheep) end)
Controls.ButtonDeer:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemDeer) end)
Controls.ButtonIvory:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemIvory) end)
Controls.ButtonFur:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemFur) end)
Controls.ButtonTruffles:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemTruffles) end)
Controls.ButtonBird:RegisterCallback(Mouse.eLClick, function() OnButtonClicked(iTotemBird) end)

--[[
function OnEnterCityScreen()
  g_bWasHidden = Controls.TotemStackFrame:IsHidden()
  Controls.TotemStackFrame:SetHide(true)
end
Events.SerialEventEnterCityScreen.Add(OnEnterCityScreen)

function OnExitCityScreen()
  Controls.TotemStackFrame:SetHide(g_bWasHidden)
end
Events.SerialEventExitCityScreen.Add(OnExitCityScreen) ]]

function ShowHideHandler(bIsHide, bInitState)
	if (not bInitState) then
		pCity = load(activePlayer,"tSamniteCity")
		if (not bIsHide) then
			Controls.ClanName:SetText(pCity:GetName())
			Controls.TotemStackFrame:SetHide(false)
		end
	end
end
ContextPtr:SetShowHideHandler(ShowHideHandler)
ContextPtr:SetHide(true)
--=======================================================================================================================
--TROLLER'S ADDITIONAL ACHIEVEMENTS
--=======================================================================================================================
include("AdditionalAchievementsUtility.lua");
local TAAid = "432bc547-eb05-4189-9e46-232dbde8f09f"
local isAAActive = Game_IsUsingMod(TAAid)
----------------------------------------
-- Unlocking prerequisites
local HasConqueredRomanCity = false
local ThreeSamnitesRomeWars = false

function SamnitesHasConqueredRomanCity(oldOwnerID, capital, plotX, plotY, newOwnerID)
	local pPlayer = Players[newOwnerID]
	local pOtherPlayer = Players[oldOwnerID]
	local civDesc = Locale.ConvertTextKey( GameInfo.Civilizations[pOtherPlayer:GetCivilizationType()].Description )
	local civShortDesc = Locale.ConvertTextKey( GameInfo.Civilizations[pOtherPlayer:GetCivilizationType()].ShortDescription )
	if HasConqueredRomanCity then return end
	if (HasTrait(pPlayer, traitSamnitesID) and pPlayer:IsAlive()) then
		local bcheck1 = string.match(civDesc, "Roman") or string.match(civDesc, "roman") or string.match(civShortDesc, "Rome") or string.match(civShortDesc, "rome")
		local bcheck2 = string.match(civDesc, "Holy") or string.match(civDesc, "holy") or string.match(civShortDesc, "Holy") or string.match(civShortDesc, "holy")
		if bcheck1 and not bcheck2 then
			HasConqueredRomanCity = true
		end
	end
end

function UnlockSamniteAchievement(iPlayer)
	local pPlayer = Players[iPlayer]
	if IsAAUnlocked('AA_JAR_SAMNITES_SPECIAL') then return end
	if (HasTrait(pPlayer, traitSamnitesID) and pPlayer:IsAlive()) then
		for i = 0, GameDefines.MAX_CIV_PLAYERS - 1 do 
			if i ~= playerID then 
				local pOtherPlayer = Players[i]
				local civDesc = Locale.ConvertTextKey( GameInfo.Civilizations[pOtherPlayer:GetCivilizationType()].Description )
				local civShortDesc = Locale.ConvertTextKey( GameInfo.Civilizations[pOtherPlayer:GetCivilizationType()].ShortDescription )
				if pPlayer:GetNumWarsFought(i) >= 3 then
					local bcheck1 = string.match(civDesc, "Roman") or string.match(civDesc, "roman") or string.match(civShortDesc, "Rome") or string.match(civShortDesc, "rome")
					local bcheck2 = string.match(civDesc, "Holy") or string.match(civDesc, "holy") or string.match(civShortDesc, "Holy") or string.match(civShortDesc, "holy")
					if bcheck1 and not bcheck2 then
						ThreeSamnitesRomeWars = true
					end				
				end
			end
		end
		if ThreeSamnitesRomeWars and HasConqueredRomanCity then
			UnlockAA('AA_JAR_SAMNITES_SPECIAL')
		end
	end
end
if isAAActive then
	GameEvents.CityCaptureComplete.Add(SamnitesHasConqueredRomanCity)
	GameEvents.PlayerDoTurn.Add(UnlockSamniteAchievement)
end
---------------------------------------------------------
print("Samnites functions loaded")
--==========================================================================================================================
--==========================================================================================================================