-- Samnites_MTFunctions
-- Author: Jarcast
-- DateCreated: 11/22/2021 12:28:17 PM
--=======================================================================================================================
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
WARN_NOT_SHARED = false; include( "SaveUtils" ); MY_MOD_NAME = "Jarcast_Samnites2";

include("FLuaVector.lua")
include("IconSupport.lua")
include("InstanceManager.lua")
include("PlotIterators.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local Teams 					= Teams
local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function Neirai_GetNearestCity(pPlayer, pPlot)
    local distance = 9999
    local cNearestCity = nil
    for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pPlot:GetX(), pPlot:GetY())
        if between < distance then
            distance = between
            cNearestCity = cCity
        end
    end
    return cNearestCity
end
-------------------------------------------------------------------------------------------------------------------------
function GetBuildTimeFromImprovementID(improvID)
	for row in DB.Query("Select a.ID ImprovID, a.Type ImprovTipe, b.Time BuildTime, b.Type from Improvements a, Builds b WHERE a.Type = b.ImprovementType and a.ID = '" .. improvID .. "'") do
		return row.BuildTime
	end
end
--=======================================================================================================================
-- Globals
--=======================================================================================================================
local iCiv = GameInfoTypes.CIVILIZATION_SAMNITES
local traitSamnitesID = GameInfoTypes["TRAIT_SAMNITES"]
local g_IsTraitActive = IsTraitActive(traitSamnitesID)
local iMeddisToutiks = GameInfoTypes["UNIT_MEDDIS_TOUTIKS"]
local iUUClass = GameInfo.UnitClasses[GameInfo.Units[iMeddisToutiks].Class].ID
local missionMeddisToutiks = GameInfoTypes["MISSION_JAR_MEDDISTOUTIKS"]
local CUSTOM_MISSION_NO_ACTION		 	= 0
local CUSTOM_MISSION_ACTION			 	= 1
local CUSTOM_MISSION_DONE            	= 2
local CUSTOM_MISSION_ACTION_AND_DONE 	= 3

local UUProd_EnemyKilled = 1
local UUProd_ImprovBuilt = 1
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
--=======================================================================================================================
-- UU: Meddis Totiks
--=======================================================================================================================
JarSamnites_BuildFinished_PlayerID = 0
JarSamnites_BuildFinished_X = 0
JarSamnites_BuildFinished_Y = 0
JarSamnites_BuildFinished_Improvement = 0

function Jar_UUProd_ImprovementBuilt(playerID, plotX, plotY, improvementID)
	if(JarSamnites_BuildFinished_PlayerID == playerID and JarSamnites_BuildFinished_X == plotX and JarSamnites_BuildFinished_Y == plotY and JarSamnites_BuildFinished_Improvement == improvementID) then
		return;
	end
	-- Store inputs for duplication check
	JarSamnites_BuildFinished_PlayerID = playerID;	
	JarSamnites_BuildFinished_X = plotX;					
	JarSamnites_BuildFinished_Y = plotY;					
	JarSamnites_BuildFinished_Improvement = improvementID;
	
	local buildTurns = GetBuildTimeFromImprovementID(improvementID) /100

	local pPlot = Map.GetPlot(plotX, plotY)
	local pPlayer = Players[playerID]
	local iEra = pPlayer:GetCurrentEra()
	local factor = (iEra+1)/2
	local bGeneral = pPlayer:GetUnitClassCount(iUUClass) > 0
	if not bGeneral then return end
	local bMeddis = false
	for loopUnit in pPlayer:Units() do
		if loopUnit:GetUnitType() == iMeddisToutiks then
			bMeddis = true
			break
		end
	end
	if bMeddis then
		for adjacentPlot in PlotAreaSweepIterator(pPlot, 2, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
			if adjacentPlot:IsUnit() then
				for i = 0, adjacentPlot:GetNumUnits() - 1 do
					local pUnit = adjacentPlot:GetUnit(i)
					if pUnit:GetUnitType() == iMeddisToutiks then
						MeddisProdStored =  load( pUnit, "MeddisProdStored" );
						if MeddisProdStored == nil then MeddisProdStored = 0 end
						local newvalue = MeddisProdStored + math.ceil(factor*buildTurns*iMod*UUProd_ImprovBuilt)
						--print("Stored prod: " .. newvalue)
						save( pUnit, "MeddisProdStored", newvalue)					
						break
					end
				end
			end
		end
	end
end

if g_IsTraitActive then GameEvents.BuildFinished.Add(Jar_UUProd_ImprovementBuilt) end

function Jar_UUProd_EnemyKilled(iOwner, iUnitID, iUnitType, iX, iY, bDelay, iKiller)
	if iKiller == -1 then return end
	local pPlayer = Players[iKiller]
	local oPlayer = Players[iOwner]
	local bMeddis = false
	local bGeneral = pPlayer:GetUnitClassCount(iUUClass) > 0
	if not bGeneral then return end
	for loopUnit in pPlayer:Units() do
		if loopUnit:GetUnitType() == iMeddisToutiks then
			bMeddis = true
			break
		end
	end
	if not bMeddis then return end
	if bDelay == true then
		local oUnit = oPlayer:GetUnitByID(iUnitID)
		if not oUnit then return end
		if not oUnit:IsCombatUnit()then return end
		local unitplot = oUnit:GetPlot()
		local infoUnit = GameInfo.Units[iUnitType]
		local iCombat = math.max( infoUnit.Combat , infoUnit.RangedCombat )
		for adjacentPlot in PlotAreaSweepIterator(unitplot, 2, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
			if adjacentPlot:IsUnit() then
				for i = 0, adjacentPlot:GetNumUnits() - 1 do
					local pUnit = adjacentPlot:GetUnit(i)
					if pUnit:GetUnitType() == iMeddisToutiks then
						MeddisProdStored =  load( pUnit, "MeddisProdStored" );
						if MeddisProdStored == nil then MeddisProdStored = 0 end
						local prodBonus = 10*math.ceil(UUProd_EnemyKilled*iCombat/10)
						local newvalue = MeddisProdStored + prodBonus
						save( pUnit, "MeddisProdStored", newvalue)						
						local iX = pUnit:GetX()
						local iY = pUnit:GetY()
						if pPlayer:IsHuman() then
							Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(iX, iY))), "+" .. prodBonus .. " [ICON_PRODUCTION]" , 2)
						end
						break
					end
				end
			end
		end
	end
end		
if g_IsTraitActive then GameEvents.UnitPrekill.Add(Jar_UUProd_EnemyKilled) end;

function Jar_UUCanHurryProduction(unit) -- checks if a unit is on a valid plot
	local pBool = false;
	local pPlot = unit:GetPlot();
	MeddisProdStored =  load( unit, "MeddisProdStored" );
	if MeddisProdStored == nil or MeddisProdStored == 0 then return pBool end
	if unit:MovesLeft() <= 1 then return pBool end
	if unit:GetOwner() ~= pPlot:GetOwner() then return pBool end
	if pPlot ~= nil then
		if pPlot:IsCity() then
			local pCity = pPlot:GetPlotCity();
			local bProcess = pCity:IsProduction() and not pCity:IsProductionProcess()
			if bProcess then
				pBool = true
				print("Samnite Meddis Toutiks can act in " .. pCity:GetName())
			end
		else		
			local pCity = pPlot:GetWorkingCity()
			local bProcess = pCity:IsProduction() and not pCity:IsProductionProcess()
			if pCity and bProcess then
				for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
					local pAdjacentPlot = Map.PlotDirection(unit:GetX(), unit:GetY(), direction)
					if pAdjacentPlot and pAdjacentPlot:IsCity() then
						pBool = true
						print("Samnite Meddis Toutiks can act in " .. pCity:GetName())
						break
					end
				end
			end
		end
	end
	return pBool
end

function Jar_UUHurryProduction(pPlayer, pUnit)
	MeddisProdStored = load( pUnit, "MeddisProdStored")
	local pUnitPlot = pUnit:GetPlot();
	local pCity
	if pUnitPlot:IsCity() then
		pCity = pUnitPlot:GetPlotCity();
	else
		pCity = pUnitPlot:GetWorkingCity()
	end
	local threshold = pCity:GetProductionNeeded()
	local Prod2Add = math.min(MeddisProdStored, threshold)
	pCity:ChangeProduction(Prod2Add)
	local cityplot = pCity:Plot()
	local iX = cityplot:GetX()
	local iY = cityplot:GetY()
	if pPlayer:IsHuman() then
		Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(iX, iY))), "+" .. Prod2Add .. " [ICON_PRODUCTION]" , 2)
	end
	local newvalue = MeddisProdStored - Prod2Add
	save( pUnit, "MeddisProdStored", newvalue)
end

function Jar_MeddisProdStored(playerID)
	local pPlayer = Players[playerID]
	local bGeneral = pPlayer:GetUnitClassCount(iUUClass) > 0
	if not bGeneral then return end
	local bMeddis = false
	for loopUnit in pPlayer:Units() do
		if loopUnit:GetUnitType() == iMeddisToutiks then
			bMeddis = true
			break
		end
	end
	if not bMeddis then return end
	local tTextstring = {}
	for pUnit in pPlayer:Units() do
		if pUnit:GetUnitType() == iMeddisToutiks then
			MeddisProdStored =  load( pUnit, "MeddisProdStored" );
			if MeddisProdStored == nil then MeddisProdStored = 0 end
			local pUnitPlot = pUnit:GetPlot();
			local place = ""
			local plotOwner = pUnitPlot:GetOwner()
			if plotOwner ~= -1 then
				local pPlotOwner = Players[plotOwner]
				local pCity = Neirai_GetNearestCity(pPlotOwner, pUnitPlot)
				place = " (" .. pCity:GetName() .. ", "
				if pPlotOwner:IsMinorCiv() then
					place = place .. Locale.ConvertTextKey( GameInfo.MinorCivilizations[pPlotOwner:GetMinorCivType()].Description ) .. ")"
				else
					place = place .. Locale.ConvertTextKey( GameInfo.Civilizations[pPlotOwner:GetCivilizationType()].ShortDescription ) .. ")"
				end
			else
				place = " (unclaimed land)"
			end
			local textstr = pUnit:GetName() .. place .. ": +" .. MeddisProdStored .. "[ICON_PRODUCTION][NEWLINE]"
			table.insert(tTextstring, textstr)
		end
	end		
	-- Notification
	local sTitle = "Meddis Toutiks Stored [ICON_PRODUCTION]"
	local sText = ""
	local icheck = 0
	for k, v in pairs(tTextstring) do
		sText = sText .. k .. ") " .. v
		icheck = k
	end
	if pPlayer:IsHuman() and icheck>0 then
		pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, sTitle) --, ElectedPlotX, ElectedPlotY)
	end
end

if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_MeddisProdStored) end


function Jar_AIMeddisProdStored(playerID)
	local aPlayer = Players[playerID]
	local aTeam = Teams[aPlayer:GetTeam()]
	if not aPlayer:IsAlive() then return end
	if aPlayer:IsHuman() then return end
	local bGeneral = aPlayer:GetUnitClassCount(iUUClass) > 0
	if not bGeneral then return end
	local bMeddis = false
	for loopUnit in aPlayer:Units() do
		if loopUnit:GetUnitType() == iMeddisToutiks then
			bMeddis = true
			break
		end
	end
	if not bMeddis then return end
	for aUnit in aPlayer:Units() do
		if aUnit:GetUnitType() == iMeddisToutiks then
			MeddisProdStored =  load( aUnit, "MeddisProdStored" );
			if MeddisProdStored == nil then MeddisProdStored = 0 end
			if MeddisProdStored > 0 then
				local aUnitPlot = aUnit:GetPlot();
				local aCity = Neirai_GetNearestCity(aPlayer, aUnitPlot)
				local threshold = aCity:GetProductionNeeded()
				local Prod2Add = math.min(MeddisProdStored, threshold)
				aCity:ChangeProduction(Prod2Add)
				local newvalue = MeddisProdStored - Prod2Add
				save( aUnit, "MeddisProdStored", newvalue)
			end		
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AIMeddisProdStored) end
--==========================================================================================================================
-- UI FUNCTIONS
--==========================================================================================================================
-------------------------------------------------------------------------------------------------------------------------
-- UNIT PANEL
-------------------------------------------------------------------------------------------------------------------------
local EUI = (MapModData and MapModData.UI_bc1)

if (not isUsingCBP) then
--Jar_MeddisToutiks_UI_UnitActionButton
function Jar_MeddisToutiks_UI_UnitActionButton()
	local unit = UI.GetHeadSelectedUnit();
	Jar_UUHurryProduction(activePlayer, unit)
end

--Jar_MeddisToutiks_UI_SerialEventUnitInfoDirty
function Jar_MeddisToutiks_UI_SerialEventUnitInfoDirty()
	local unit = UI.GetHeadSelectedUnit();
	if (not unit) then return end
	if unit:GetUnitType() == iMeddisToutiks then
		local bcheck = not Jar_UUCanHurryProduction(unit)
		Controls.UnitActionButton:SetHide(bcheck) -- if not allowed -> hide true; if allowed -> hide false
	else
		Controls.UnitActionButton:SetHide(true)
	end
	
	local buildCityButtonActive = unit:IsFound();				
	
	local primaryStretchy = ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStretchy")
	local primaryStack = ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStack") 
	primaryStack:CalculateSize();
	primaryStack:ReprocessAnchoring();
    
	local stackSize = primaryStack:GetSize();
	local stretchySize = primaryStretchy:GetSize();
	local buildCityButtonSize = 0
	if buildCityButtonActive then
		if OptionsManager.GetSmallUIAssets() and not UI.IsTouchScreenEnabled() then
			buildCityButtonSize = 36;
		else
			buildCityButtonSize = 60;
		end
	end
	primaryStretchy:SetSizeVal( stretchySize.x, stackSize.y + buildCityButtonSize + 348 );
end

--Jar_MeddisToutiks_UI_UpdateUnitInfoPanel
local function Jar_MeddisToutiks_UI_UpdateUnitInfoPanel()
	if (not OptionsManager.GetSmallUIAssets()) then
		Controls.UnitActionButton:SetSizeVal(50,50)
		Controls.UnitActionIcon:SetSizeVal(64,64)
		Controls.UnitActionIcon:SetTexture("Jar_Meddis_MissionAtlas_64.dds")
	else
		Controls.UnitActionButton:SetSizeVal(36,36)
		Controls.UnitActionIcon:SetSizeVal(45,45)
		Controls.UnitActionIcon:SetTexture("Jar_Meddis_MissionAtlas_45.dds")
	end
	Controls.UnitActionIcon:LocalizeAndSetToolTip("TXT_KEY_JAR_MEDDIS_HURRY_PROD")
	Controls.UnitActionButton:ChangeParent(ContextPtr:LookUpControl("/InGame/WorldView/UnitPanel/PrimaryStack"))
end

local function Initialize()	
	Events.LoadScreenClose.Add(Jar_MeddisToutiks_UI_UpdateUnitInfoPanel);
	Events.SerialEventUnitInfoDirty.Add(Jar_MeddisToutiks_UI_SerialEventUnitInfoDirty);
end
IconHookup(0, 45, "JAR_SAMNITES_MISSION_ATLAS", Controls.UnitActionIcon)	
Controls.UnitActionButton:RegisterCallback(Mouse.eLClick, Jar_MeddisToutiks_UI_UnitActionButton);
Initialize();

if EUI then
--==========================================================================================================================
-- ENHANCED USER INTERFACE COMPATIBILITY
--==========================================================================================================================
IconHookup(0, 45, "JAR_SAMNITES_MISSION_ATLAS", Controls.TomButtonImage)
Controls.TomButtonBackground:SetHide(true)
Controls.UnitActionButton:SetHide(true)
local pSelUnit
function Jar_MeddisToutiksSelection(playerID, unitID, x, y, a5, bool)
	if bool then
		local pPlayer = Players[playerID]
		local pUnit = pPlayer:GetUnitByID(unitID);
		if pUnit:GetUnitType() == iMeddisToutiks then
			ContextPtr:SetHide(false)
			Controls.TomButtonBackground:SetHide(false)
			pSelUnit = pUnit;
			Controls.TomButton:SetDisabled(false)
			Controls.TomButton:RegisterCallback(Mouse.eLClick, Jar_DoPerformAction)	
			Controls.TomButton:LocalizeAndSetToolTip("TXT_KEY_JAR_MEDDIS_HURRY_PROD")
		end
	else
		Controls.TomButtonBackground:SetHide(true)
		pSelUnit = nil;
	end
end
Events.UnitSelectionChanged.Add(Jar_MeddisToutiksSelection)

function Jar_DoPerformAction(playerID, unitID)
	local hPlayer = Players[playerID]
	local hTeam = hPlayer:GetTeam()
	local hUnit = pSelUnit;
	local hUnitPlot = hUnit:GetPlot();
	if Jar_UUCanHurryProduction(hUnit) then
		Jar_UUHurryProduction(hPlayer, hUnit)
	else
		local iX = hUnit:GetX()
		local iY = hUnit:GetY()
		Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(iX, iY))), "[COLOR_WARNING_TEXT]" .. Locale.ConvertTextKey("TXT_KEY_MISSION_JAR_MEDDIS_HURRY_PROD_DISABLED") .. "[ENDCOLOR]", 0)
	end
end
end
else
--==========================================================================================================================
-- CBP MISSION EVENTS COMPATIBILITY
--==========================================================================================================================
-- Jar_MeddisToutiks_CustomMissionStart
local function Jar_MeddisToutiks_CustomMissionStart(playerID, unitID, missionID, data1, data2, flags, turn)
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	local plot = unit:GetPlot()
	if missionID == missionMeddisToutiks then
		Jar_UUHurryProduction(player, unit)
		return CUSTOM_MISSION_ACTION
	end
	return CUSTOM_MISSION_NO_ACTION
end

-- Jar_MeddisToutiks_CustomMissionCompleted
local function Jar_MeddisToutiks_CustomMissionCompleted(playerID, unitID, missionID, data1, data2, flags, turn)
	return (missionID == missionMeddisToutiks)
end

-- Jar_MeddisToutiks_CustomMissionPossible
local function Jar_MeddisToutiks_CustomMissionPossible(playerID, unitID, missionID, data1, data2, _, _, plotX, plotY, bTestVisible)
	if ((not playerID) or (not unitID) or (not missionID)) then return end
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	if missionID == missionMeddisToutiks then
		if unit:GetUnitType() == iMeddisToutiks then
			return Jar_UUCanHurryProduction(unit)
		end
	end	
	return false
end

GameEvents.CustomMissionStart.Add(Jar_MeddisToutiks_CustomMissionStart)
GameEvents.CustomMissionCompleted.Add(Jar_MeddisToutiks_CustomMissionCompleted)
GameEvents.CustomMissionPossible.Add(Jar_MeddisToutiks_CustomMissionPossible)
end

print("Samnites MT functions loaded")
--==========================================================================================================================
--==========================================================================================================================